#include <QtGui>
#include <iostream>
#include <cmath>
#include "bsaccumulatoritem.h"

using namespace std;



const int BSAccumulatorItem::DEFAULT_PEN_WIDTH = 1;
const int BSAccumulatorItem::LEFT_MARGIN = 16;
const int BSAccumulatorItem::TEXT_HEIGHT = 16;


BSAccumulatorItem::BSAccumulatorItem (BSDetector *sd, int w, int h)
{
  detector = sd;
  width = w;
  height = h;
  selaccu = (sd->isFiltering (BSDetector::STEP_FINAL) ?
             BSDetector::STEP_FINAL : BSDetector::STEP_INITIAL);
  map[0] = NULL;
  map[1] = NULL;
  solution[0] = -1;
  solution[1] = -1;
  resol[0] = 1.;
  resol[1] = 1.;
  mask = NULL;
  verbose = true;
  infoPen = QPen (Qt::red, DEFAULT_PEN_WIDTH, Qt::SolidLine,
                  Qt::RoundCap, Qt::RoundJoin);
  scalePen = QPen (Qt::red, 2);
  markPen = QPen (Qt::blue, 4);
}


BSAccumulatorItem::~BSAccumulatorItem ()
{
  delete [] map[0];
  delete [] map[1];
}


QRectF BSAccumulatorItem::boundingRect () const
{
  return QRectF (0, 0, width, height);
}


void BSAccumulatorItem::paint (QPainter *painter,
                               const QStyleOptionGraphicsItem *option,
                               QWidget *widget)
{
  Q_UNUSED (option);
  Q_UNUSED (widget);

  if (! (detector->getBlurredSegments().empty ()
         && detector->isFiltering (selaccu)))
  {
    painter->fillRect (0, 0, width, height, QBrush (Qt::black));
    return;
  }

  // Updates the grid
  LineSpaceFilter *hugues = detector->getFilter (selaccu);
  int nbc = hugues->width ();
  int nbl = hugues->height ();
  int cw = width / nbc;
  int ch = height / nbl;

  // Updates the accumulator
  if (map[selaccu] == NULL) map[selaccu] = new uchar[nbc * nbl];
  hugues->getAccumulator (map[selaccu], solution);
  resol[0] = hugues->angularResolution () * 180. / M_PI;
  resol[1] = hugues->distanceResolution ();
  mask = hugues->getMask ();

  // Draws the accumulator
  if (solution[0] >= 0)
  {
    uchar *pix = map[selaccu];
    for (int i = 0; i < nbc * nbl; i++, pix++)
      painter->fillRect ((i % nbc) * cw, (nbl - 1 - i / nbc) * ch, cw, ch,
                         QBrush (QColor ((int) *pix, (int) *pix, (int) *pix)));

    // Marks the filtering cells
    if (mask != NULL)
    {
      painter->setPen (markPen);
      bool *maski = mask;
      for (int j = 0; j < nbl; j++)
        for (int i = 0; i < nbc; i++)
          if (*maski++)
            painter->drawPoint ((i + 0.5) * cw, (nbl - j - 0.5) * ch);
    }

    if (verbose)
    {
      // Displays the scale (one radian / one pixel)
      painter->setPen (scalePen);
      int sh = (int) (ch / resol[1] + 0.5);
      painter->drawRect (0, height - 1 - sh, (int) (cw / resol[0] + 0.5), sh);

      // Displays information
      initText (painter);
      addText (painter, QString ("S size : ") + QString::number (nbc)
                        + QString ("x") + QString::number (nbl));
      addText (painter, QString ("R resolution : ")
                        + QString::number (resol[0]) + QString (" pixels x ")
                        + QString::number (resol[1]) + QString (" degrees"));
      addText (painter, QString ("P sub-pixellisation level : ")
                        + QString::number (hugues->getSubpix ()));
      addText (painter, QString ("F selectivity : ")
                        + QString::number (hugues->getSelectionThreshold ())
                        + QString ("% of max vote"));
    }
  }
}


bool BSAccumulatorItem::resizeAccumulator (bool larger)
{
  if (! detector->isFiltering (selaccu)) return false;
  LineSpaceFilter *hugues = detector->getFilter (selaccu);
  bool res = hugues->resize (larger);
  if (res)
  {
    delete [] map[selaccu];
    map[selaccu] = new uchar[hugues->width () * hugues->height ()];
  }
  return (res);
}


bool BSAccumulatorItem::zoomAccumulator (bool in)
{
  if (! detector->isFiltering (selaccu)) return false;
  LineSpaceFilter *hugues = detector->getFilter (selaccu);
  return (hugues->zoom (in));
}


bool BSAccumulatorItem::subpixellise (bool larger)
{
  if (! detector->isFiltering (selaccu)) return false;
  LineSpaceFilter *hugues = detector->getFilter (selaccu);
  return (hugues->subpixellise (larger));
}


bool BSAccumulatorItem::setFilterSelectivity (bool larger)
{
  if (! detector->isFiltering (selaccu)) return false;
  LineSpaceFilter *hugues = detector->getFilter (selaccu);
  return (hugues->setSelectivity (larger));
}


void BSAccumulatorItem::initText (QPainter *painter)
{
  painter->setPen (infoPen);
  textOffset = 0;
}


void BSAccumulatorItem::addText (QPainter *painter, const QString &text)
{
  textOffset += TEXT_HEIGHT;
  painter->drawText (LEFT_MARGIN, textOffset, text);
}
